@@ -21,6 +21,7 @@
 #include "lib/hmapx.h"
 #include "openvswitch/hmap.h"
 #include "openvswitch/shash.h"
+#include "lib/ovn-util.h"
 #include "lib/smap.h"
 #include "lib/sset.h"
 #include "lib/util.h"
@@ -103,6 +104,7 @@ vtep_create_mmr(struct ovsdb_idl_txn *vtep_idl_txn, const char *mac,
     struct vteprec_mcast_macs_remote *new_mmr =
        vteprec_mcast_macs_remote_insert(vtep_idl_txn);
 
+    VLOG_DBG("Inserting MMR for LS '%s'", vtep_ls->name);
     vteprec_mcast_macs_remote_set_MAC(new_mmr, mac);
     vteprec_mcast_macs_remote_set_logical_switch(new_mmr, vtep_ls);
     vteprec_mcast_macs_remote_set_locator_set(new_mmr, ploc_set);
@@ -139,7 +141,7 @@ vtep_process_pls(const struct ovs_list *locators_list,
                            ploc_entry->vteprec_ploc;
             if (mmr_ext && !shash_find_data(&mmr_ext->physical_locators,
                                             locators[i]->dst_ip)) {
-                    locator_lists_differ = true;
+                locator_lists_differ = true;
             }
             i++;
         }
@@ -148,8 +150,8 @@ vtep_process_pls(const struct ovs_list *locators_list,
     return locator_lists_differ;
 }
 
-/* Creates a new 'Mcast_Macs_Remote' entry if needed and also cleans up
- * out-dated remote mcast mac entries as needed. */
+/* Creates a new 'Mcast_Macs_Remote' entry or modifies existing if needed
+ * and also cleans up out-dated remote mcast mac entries as needed. */
 static void
 vtep_update_mmr(struct ovsdb_idl_txn *vtep_idl_txn,
                 struct ovs_list *locators_list,
@@ -161,22 +163,29 @@ vtep_update_mmr(struct ovsdb_idl_txn *vtep_idl_txn,
     bool mmr_changed;
 
     locators = xmalloc(n_locators_new * sizeof *locators);
-
     mmr_changed = vtep_process_pls(locators_list, mmr_ext, locators);
 
-    if (mmr_ext && !n_locators_new) {
-        vteprec_mcast_macs_remote_delete(mmr_ext->mmr);
-    } else if ((mmr_ext && mmr_changed) ||
-               (!mmr_ext && n_locators_new)) {
+    if (mmr_changed) {
+        if (n_locators_new) {
+            const struct vteprec_physical_locator_set *ploc_set =
+                vteprec_physical_locator_set_insert(vtep_idl_txn);
 
-        const struct vteprec_physical_locator_set *ploc_set =
-            vteprec_physical_locator_set_insert(vtep_idl_txn);
+            vteprec_physical_locator_set_set_locators(ploc_set, locators,
+                                                      n_locators_new);
 
-        vtep_create_mmr(vtep_idl_txn, "unknown-dst", vtep_ls, ploc_set);
+            if (!mmr_ext) {  /* create new mmr */
+                vtep_create_mmr(vtep_idl_txn, "unknown-dst", vtep_ls,
+                                ploc_set);
+            } else {  /* update existing mmr */
+                vteprec_mcast_macs_remote_set_locator_set(mmr_ext->mmr,
+                                                          ploc_set);
+            }
 
-        vteprec_physical_locator_set_set_locators(ploc_set, locators,
-                                                  n_locators_new);
+        } else if (mmr_ext) {  /* remove old mmr */
+            vteprec_mcast_macs_remote_delete(mmr_ext->mmr);
+        }
     }
+
     free(locators);
 }
 
@@ -225,18 +234,23 @@ vtep_lswitch_run(struct shash *vtep_pbs, struct sset *vtep_pswitches,
             }
 
             tnl_key = port_binding_rec->datapath->tunnel_key;
-            if (vtep_ls->n_tunnel_key
-                && vtep_ls->tunnel_key[0] != tnl_key) {
-                VLOG_DBG("set vtep logical switch (%s) tunnel key from "
-                         "(%"PRId64") to (%"PRId64")", vtep_ls->name,
-                         vtep_ls->tunnel_key[0], tnl_key);
+            if ((vtep_ls->n_tunnel_key && vtep_ls->tunnel_key[0] != tnl_key)
+                || !vtep_ls->n_tunnel_key) {
+                VLOG_DBG("set vtep logical switch (%s) tunnel key to %"PRId64,
+                         vtep_ls->name, tnl_key);
+                vteprec_logical_switch_set_tunnel_key(vtep_ls, &tnl_key, 1);
             }
-            vteprec_logical_switch_set_tunnel_key(vtep_ls, &tnl_key, 1);
 
             /* OVN is expected to always use source node replication mode,
              * hence the replication mode is hard-coded for each logical
              * switch in the context of ovn-controller-vtep. */
-            vteprec_logical_switch_set_replication_mode(vtep_ls, "source_node");
+            if (!vtep_ls->replication_mode
+                || strcmp(vtep_ls->replication_mode, "source_node")) {
+
+                vteprec_logical_switch_set_replication_mode(vtep_ls,
+                                                            "source_node");
+            }
+
             sset_add(&used_ls, lswitch_name);
         }
     }
@@ -352,22 +366,30 @@ vtep_macs_run(struct ovsdb_idl_txn *vtep_idl_txn, struct shash *ucast_macs_rmts,
             shash_add(&ls_node->physical_locators, chassis_ip, pl);
         }
 
-        char *mac_tnlkey = xasprintf("%s_%"PRId64, "unknown-dst", tnl_key);
-        ls_node->mmr_ext = shash_find_data(mcast_macs_rmts, mac_tnlkey);
+        if (!ls_node->mmr_ext) {
+            char *mac_tnlkey = xasprintf("%s_%"PRId64, "unknown-dst", tnl_key);
+            ls_node->mmr_ext = shash_find_data(mcast_macs_rmts, mac_tnlkey);
 
-        if (ls_node->mmr_ext &&
-            ls_node->mmr_ext->mmr->logical_switch == ls_node->vtep_ls) {
+            if (ls_node->mmr_ext &&
+                ls_node->mmr_ext->mmr->logical_switch == ls_node->vtep_ls) {
 
-            /* Delete the entry from the hash table so the mmr does not get
-             * removed from the DB later on during stale checking. */
-            shash_find_and_delete(mcast_macs_rmts, mac_tnlkey);
+                /* Delete the entry from the hash table so the mmr does not get
+                * removed from the DB later on during stale checking. */
+                shash_find_and_delete(mcast_macs_rmts, mac_tnlkey);
+            }
+            free(mac_tnlkey);
         }
-        free(mac_tnlkey);
 
         for (i = 0; i < port_binding_rec->n_mac; i++) {
             const struct vteprec_ucast_macs_remote *umr;
             const struct sbrec_port_binding *conflict;
-            char *mac = port_binding_rec->mac[i];
+            struct lport_addresses laddrs;
+
+            if (!extract_lsp_addresses(port_binding_rec->mac[i], &laddrs)) {
+                continue;
+            };
+
+            char *mac = laddrs.ea_s;
 
             /* Checks for duplicate MAC in the same vtep logical switch. */
             conflict = shash_find_data(&ls_node->added_macs, mac);
@@ -385,7 +407,7 @@ vtep_macs_run(struct ovsdb_idl_txn *vtep_idl_txn, struct shash *ucast_macs_rmts,
                                             tnl_key);
             umr = shash_find_data(ucast_macs_rmts, mac_ip_tnlkey);
             /* If finds the 'umr' entry for the mac, ip, and tnl_key, deletes
-             * the entry from shash so that it is not gargage collected.
+             * the entry from shash so that it is not garbage collected.
              *
              * If not found, creates a new 'umr' entry. */
             if (umr && umr->logical_switch == ls_node->vtep_ls) {
@@ -396,6 +418,7 @@ vtep_macs_run(struct ovsdb_idl_txn *vtep_idl_txn, struct shash *ucast_macs_rmts,
                 vteprec_ucast_macs_remote_set_locator(new_umr, pl);
             }
             free(mac_ip_tnlkey);
+            destroy_lport_addresses(&laddrs);
         }
     }
 
@@ -527,7 +550,7 @@ vtep_run(struct controller_vtep_ctx *ctx)
                       mmr->logical_switch && mmr->logical_switch->n_tunnel_key
                           ? mmr->logical_switch->tunnel_key[0] : INT64_MAX);
 
-        shash_add(&mcast_macs_rmts, mac_tnlkey, mmr_ext);
+        shash_add_once(&mcast_macs_rmts, mac_tnlkey, mmr_ext);
         mmr_ext->mmr = mmr;
 
         shash_init(&mmr_ext->physical_locators);
