@@ -59,6 +59,7 @@ struct tunnel_ctx {
 
     struct ovsdb_idl_txn *ovs_txn;
     const struct ovsrec_bridge *br_int;
+    const struct sbrec_chassis *this_chassis;
 };
 
 struct chassis_node {
@@ -176,6 +177,28 @@ tunnel_add(struct tunnel_ctx *tc, const struct sbrec_sb_global *sbg,
 
     /* Add auth info if ipsec is enabled. */
     if (sbg->ipsec) {
+        const struct sbrec_chassis *this_chassis = tc->this_chassis;
+        const char *local_ip = NULL;
+
+        /* Determine 'ovn-encap-ip' of the local chassis as this will be the
+         * tunnel port's 'local_ip'. We do not support the case in which
+         * 'ovn-encap-ip' holds multiple comma-delimited IP addresses.
+         */
+        for (int i = 0; i < this_chassis->n_encaps; i++) {
+            if (local_ip && strcmp(local_ip, this_chassis->encaps[i]->ip)) {
+                VLOG_ERR("ovn-encap-ip has been configured as a list. This "
+                         "is unsupported for IPsec.");
+                /* No need to loop further as we know this condition has been
+                 * hit */
+                break;
+            } else {
+                local_ip = this_chassis->encaps[i]->ip;
+            }
+        }
+
+        if (local_ip) {
+            smap_add(&options, "local_ip", local_ip);
+        }
         smap_add(&options, "remote_name", new_chassis_id);
     }
 
@@ -225,15 +248,26 @@ exit:
     smap_destroy(&options);
 }
 
-struct sbrec_encap *
-preferred_encap(const struct sbrec_chassis *chassis_rec)
+static bool
+chassis_has_type(const struct sbrec_chassis *chassis, uint32_t tun_type) {
+    for (size_t i = 0; i < chassis->n_encaps; i++) {
+        if (get_tunnel_type(chassis->encaps[i]->type) == tun_type) {
+            return true;
+        }
+    }
+    return false;
+}
+
+static struct sbrec_encap *
+preferred_encap(const struct sbrec_chassis *chassis_rec,
+                const struct sbrec_chassis *this_chassis)
 {
     struct sbrec_encap *best_encap = NULL;
     uint32_t best_type = 0;
 
-    for (int i = 0; i < chassis_rec->n_encaps; i++) {
+    for (size_t i = 0; i < chassis_rec->n_encaps; i++) {
         uint32_t tun_type = get_tunnel_type(chassis_rec->encaps[i]->type);
-        if (tun_type > best_type) {
+        if (tun_type > best_type && chassis_has_type(this_chassis, tun_type)) {
             best_type = tun_type;
             best_encap = chassis_rec->encaps[i];
         }
@@ -247,9 +281,11 @@ preferred_encap(const struct sbrec_chassis *chassis_rec)
  * as there are VTEP of that type (differentiated by remote_ip) on that chassis.
  */
 static int
-chassis_tunnel_add(const struct sbrec_chassis *chassis_rec, const struct sbrec_sb_global *sbg, struct tunnel_ctx *tc)
+chassis_tunnel_add(const struct sbrec_chassis *chassis_rec,
+                   const struct sbrec_sb_global *sbg, struct tunnel_ctx *tc,
+                   const struct sbrec_chassis *this_chassis)
 {
-    struct sbrec_encap *encap = preferred_encap(chassis_rec);
+    struct sbrec_encap *encap = preferred_encap(chassis_rec, this_chassis);
     int tuncnt = 0;
 
     if (!encap) {
@@ -310,7 +346,8 @@ encaps_run(struct ovsdb_idl_txn *ovs_idl_txn,
     struct tunnel_ctx tc = {
         .chassis = SHASH_INITIALIZER(&tc.chassis),
         .port_names = SSET_INITIALIZER(&tc.port_names),
-        .br_int = br_int
+        .br_int = br_int,
+        .this_chassis = this_chassis
     };
 
     tc.ovs_txn = ovs_idl_txn;
@@ -366,7 +403,7 @@ encaps_run(struct ovsdb_idl_txn *ovs_idl_txn,
                 continue;
             }
 
-            if (chassis_tunnel_add(chassis_rec, sbg, &tc) == 0) {
+            if (chassis_tunnel_add(chassis_rec, sbg, &tc, this_chassis) == 0) {
                 VLOG_INFO("Creating encap for '%s' failed", chassis_rec->name);
                 continue;
             }
