@@ -413,6 +413,7 @@ expr_from_node(const struct ovs_list *node)
 
 void expr_format(const struct expr *, struct ds *);
 void expr_print(const struct expr *);
+size_t expr_size(const struct expr *);
 struct expr *expr_parse(struct lexer *, const struct shash *symtab,
                         const struct shash *addr_sets,
                         const struct shash *port_groups,
@@ -437,7 +438,7 @@ struct expr *expr_evaluate_condition(
     struct expr *,
     bool (*is_chassis_resident)(const void *c_aux,
                                 const char *port_name),
-    const void *c_aux, bool *condition_present);
+    const void *c_aux);
 struct expr *expr_normalize(struct expr *);
 
 bool expr_honors_invariants(const struct expr *);
@@ -477,6 +478,7 @@ uint32_t expr_to_matches(const struct expr *,
                          const void *aux,
                          struct hmap *matches);
 void expr_matches_destroy(struct hmap *matches);
+size_t expr_matches_prepare(struct hmap *matches, uint32_t conj_id_ofs);
 void expr_matches_print(const struct hmap *matches, FILE *);
 
 /* Action parsing helper. */
@@ -543,9 +545,11 @@ void expr_constant_set_destroy(struct expr_constant_set *cs);
  * are ignored.
  */
 
-void expr_const_sets_add(struct shash *const_sets, const char *name,
-                         const char * const *values, size_t n_values,
-                         bool convert_to_integer);
+void expr_const_sets_add_integers(struct shash *const_sets, const char *name,
+                                  const char * const *values, size_t n_values);
+void expr_const_sets_add_strings(struct shash *const_sets, const char *name,
+                                 const char * const *values, size_t n_values,
+                                 const struct sset *filter);
 void expr_const_sets_remove(struct shash *const_sets, const char *name);
 void expr_const_sets_destroy(struct shash *const_sets);
 
