@@ -168,6 +168,7 @@ create_gen_opts(struct hmap *dhcp_opts, struct hmap *dhcpv6_opts,
     dhcp_opt_add(dhcp_opts, "dns_server", 6, "ipv4");
     dhcp_opt_add(dhcp_opts, "log_server", 7, "ipv4");
     dhcp_opt_add(dhcp_opts, "lpr_server",  9, "ipv4");
+    dhcp_opt_add(dhcp_opts, "hostname", 12, "str");
     dhcp_opt_add(dhcp_opts, "domain_name", 15, "str");
     dhcp_opt_add(dhcp_opts, "swap_server", 16, "ipv4");
     dhcp_opt_add(dhcp_opts, "policy_filter", 21, "ipv4");
@@ -193,6 +194,8 @@ create_gen_opts(struct hmap *dhcp_opts, struct hmap *dhcpv6_opts,
     dhcp_opt_add(dhcp_opts, "domain_search_list", 119, "domains");
     dhcp_opt_add(dhcp_opts, "bootfile_name_alt", 254, "str");
     dhcp_opt_add(dhcp_opts, "broadcast_address", 28, "ipv4");
+    dhcp_opt_add(dhcp_opts, "netbios_name_server", 44, "ipv4");
+    dhcp_opt_add(dhcp_opts, "netbios_node_type", 46, "uint8");
 
     /* DHCPv6 options. */
     hmap_init(dhcpv6_opts);
@@ -225,10 +228,10 @@ create_addr_sets(struct shash *addr_sets)
     };
     static const char *const addrs4[] = { NULL };
 
-    expr_const_sets_add(addr_sets, "set1", addrs1, 3, true);
-    expr_const_sets_add(addr_sets, "set2", addrs2, 3, true);
-    expr_const_sets_add(addr_sets, "set3", addrs3, 3, true);
-    expr_const_sets_add(addr_sets, "set4", addrs4, 0, true);
+    expr_const_sets_add_integers(addr_sets, "set1", addrs1, 3);
+    expr_const_sets_add_integers(addr_sets, "set2", addrs2, 3);
+    expr_const_sets_add_integers(addr_sets, "set3", addrs3, 3);
+    expr_const_sets_add_integers(addr_sets, "set4", addrs4, 0);
 }
 
 static void
@@ -241,8 +244,8 @@ create_port_groups(struct shash *port_groups)
     };
     static const char *const pg2[] = { NULL };
 
-    expr_const_sets_add(port_groups, "0_pg1", pg1, 3, false);
-    expr_const_sets_add(port_groups, "0_pg_empty", pg2, 0, false);
+    expr_const_sets_add_strings(port_groups, "0_pg1", pg1, 3, NULL);
+    expr_const_sets_add_strings(port_groups, "0_pg_empty", pg2, 0, NULL);
 }
 
 static bool
@@ -316,7 +319,7 @@ test_parse_expr__(int steps)
             if (steps > 1) {
                 expr = expr_simplify(expr);
                 expr = expr_evaluate_condition(expr, is_chassis_resident_cb,
-                                               &ports, NULL);
+                                               &ports);
             }
             if (steps > 2) {
                 expr = expr_normalize(expr);
@@ -920,7 +923,7 @@ test_tree_shape_exhaustively(struct expr *expr, struct shash *symtab,
             modified = expr_simplify(expr_clone(expr));
             modified = expr_evaluate_condition(
                 modified, tree_shape_is_chassis_resident_cb,
-                NULL, NULL);
+                NULL);
             ovs_assert(expr_honors_invariants(modified));
 
             if (operation >= OP_NORMALIZE) {
@@ -1345,7 +1348,9 @@ test_parse_actions(struct ovs_cmdl_context *ctx OVS_UNUSED)
                 .mac_lookup_ptable = OFTABLE_MAC_LOOKUP,
                 .lb_hairpin_ptable = OFTABLE_CHK_LB_HAIRPIN,
                 .lb_hairpin_reply_ptable = OFTABLE_CHK_LB_HAIRPIN_REPLY,
-                .ct_snat_vip_ptable = OFTABLE_CT_SNAT_FOR_VIP,
+                .ct_snat_vip_ptable = OFTABLE_CT_SNAT_HAIRPIN,
+                .fdb_ptable = OFTABLE_GET_FDB,
+                .fdb_lookup_ptable = OFTABLE_LOOKUP_FDB,
             };
             struct ofpbuf ofpacts;
             ofpbuf_init(&ofpacts, 0);
